#include "stdafx.h"
#include "SerialGate.h"
#include <iostream>
#include <Winspool.h>

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}



extern "C" __declspec (dllexport) bool SerialGate::Open(int port, int baud)
{
	char COM_string[20];
	sprintf(COM_string,"\\\\.\\COM%d", port);

	m_hFile = CreateFile(COM_string, GENERIC_READ|GENERIC_WRITE, 0, NULL,						
	   OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,NULL);
	

	if(m_hFile == INVALID_HANDLE_VALUE)
	{				
		return false;
	}


	DCB dcb;
	GetCommState(m_hFile, &dcb);

	COMMTIMEOUTS CommTimeOuts;
	CommTimeOuts.ReadIntervalTimeout		 = MAXDWORD;
	CommTimeOuts.ReadTotalTimeoutMultiplier  = 0;
	CommTimeOuts.ReadTotalTimeoutConstant    = 0;
	CommTimeOuts.WriteTotalTimeoutMultiplier = 0;
	CommTimeOuts.WriteTotalTimeoutConstant   = 1000;

	SetCommTimeouts(m_hFile, &CommTimeOuts);


	dcb.ByteSize = 8;             
	dcb.Parity   = NOPARITY;        
	dcb.StopBits = ONESTOPBIT;    
	dcb.BaudRate = baud;

	SetCommState(m_hFile, &dcb);

	this->state = true;
	return true;	
}

extern "C" __declspec (dllexport) SerialGate::SerialGate()
{
	this->state = false;
}

extern "C" __declspec (dllexport) SerialGate::~SerialGate()
{
	this->Close();
}

extern "C" __declspec (dllexport) void SerialGate::Close()
{
	this->state = false;
	if( m_hFile ) CloseHandle(m_hFile);
}

extern "C" __declspec (dllexport) void SerialGate::Clean()
{
	if(!state)
		return;

	PurgeComm(m_hFile, PURGE_TXCLEAR|PURGE_RXCLEAR);
}


extern "C" __declspec (dllexport) int SerialGate::Send(char* buff, int szBuff)
{
	if(!state)
		return 0;

	if(buff == NULL || szBuff <= 0)
	{
		return 0;
	}

	DWORD lpdwBytesWrittens = 0;
	WriteFile(m_hFile, buff, szBuff, &lpdwBytesWrittens, NULL);	
	
	return lpdwBytesWrittens;
}

extern "C" __declspec (dllexport) int SerialGate::Recv(char* buff, int szBuff)
{
	if(!state)
		return 0;

	if(buff == NULL || szBuff <= 0)
	{
		return 0;
	}

	DWORD dwBytesRead = 0;	
	ReadFile(m_hFile, buff, szBuff, &dwBytesRead, NULL);	

	return dwBytesRead;
}

extern "C" __declspec (dllexport) void SerialGate::SetLine(OUT_LINES_NAME ln, bool state)
{
	if(!state)
		return ;

	unsigned char value;
	
	if(ln == DTR)
	{
		if(state)
			value = 6;
		else
			value = 5;
	}

	
	if(ln == RTS)
	{
		if(state)
			value = 4;
		else
			value = 3;
	}

	EscapeCommFunction(m_hFile, value);	
}

extern "C" __declspec (dllexport) bool SerialGate::GetLine(IN_LINES_NAME ln)
{
	if(!state)
		return 0;

	unsigned long ul = 0;

	GetCommModemStatus(m_hFile, &ul);
		
	if(ul == 0x10 && ln == CTS)
	{
		return true;
	}

	if(ul == 0x20 && ln == DSR)
	{
		return true;	
	}

	if(ul == 0x40 && ln == RING)
	{
		return true;
	}

	if(ul == 0x80 && ln == RLSD)
	{
		return true;
	}		

	return false;
}


extern "C" __declspec (dllexport) void SerialGate::GetPortsInfo(PortInfo* pi)
{

  DWORD  Ports_MemSize = 0;
  DWORD  Ports_Count   = 0;
  BYTE*  lpPorts       = NULL;

  //Getting Ports_MemSize value...
  EnumPorts(NULL,
            1,
            lpPorts,
            0,
            &Ports_MemSize,
            &Ports_Count);


  //Getting lpPorts...
  lpPorts = new BYTE[Ports_MemSize];
  EnumPorts(NULL,
            1,
            lpPorts,
            Ports_MemSize,
            &Ports_MemSize,
            &Ports_Count);


  //Forming List Of Ports...
  DWORD dw;
  TCHAR str[8];
  char temp[4];
  int port = -1;
  PORT_INFO_1 *pPortInfo;
  pPortInfo = (PORT_INFO_1 *)lpPorts;


  int counter = 0;
  bool av = false;

  for (dw = 0; dw < Ports_Count; dw++)
  {
    
    lstrcpyn(str, pPortInfo->pName, 4);
    str[4] = 0;
    if (lstrcmpi(str, "com") == 0)
    {    
		//printf("%s\n", pPortInfo->pName);	

		memset(temp, '\0', sizeof(temp));

		temp[0] = pPortInfo->pName[3];
		if(pPortInfo->pName[4]!= ':' && pPortInfo->pName[4]!= '\0')
			temp[1] = pPortInfo->pName[4];
		if(pPortInfo->pName[5]!= ':' && pPortInfo->pName[5]!= '\0')
			temp[2] = pPortInfo->pName[5];		

		port = atoi(temp);

		//printf("%d\n", port);

		
		char COM_string[20];
		sprintf(COM_string,"\\\\.\\COM%d", port);

		HANDLE h = CreateFile(COM_string, GENERIC_READ|GENERIC_WRITE, 0, NULL,						
		OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,NULL);

		if(h == INVALID_HANDLE_VALUE)
		{				
			//return false;
			av = false;
		}
		else
		{
			av = true;
			CloseHandle(h);
		}



		pi->p[counter].Id      = port;
		pi->p[counter].Availbl = av;
		counter++;
		
		
    }

    pPortInfo++;
  }
  
  pi->koll = counter;

  delete [] lpPorts;
	
}